local tableComp = require("scenario/pseudoRandomTour/tableComp")

--- Creates a new save file
local function createSavefile()
  local content = {
    routes = {
      longestRoute = 0,
      totalDistance = 0,
      numberOfRoutes = 0
    },
    waypoints = {
      mostNumberOfWaypoints = 0,
      totalWaypoints = 0
    },
    vehicles = {
      used = {}
    }
  }

  jsonWriteFile("prt/stats.json", content, true)
end

--- Returns the savefile data.
--- Creates the savefile if it does not yet exist.
--- @return table The achievement savefile
local function openSaveFile()
  local savefile = jsonDecode(readFile("prt/stats.json"))

  if not savefile then
    createSavefile()
    -- Re-open
    savefile = jsonDecode(readFile("prt/stats.json"))
  end

  return savefile
end

--- Adds a completed scenario to the stats
--- @param routeLength number The length of the route (km)
--- @param numWaypoints number The number of waypoints in the scenario
--- @param vehicleUsed number The vehicle used (should be of the format <vehicleName>/<config>)
local function addScenarioToStats(routeLength, numWaypoints, vehicleUsed)
  local savefile = openSaveFile()

  savefile.vehicles.used[vehicleUsed] = true

  savefile.routes.longestRoute = math.max(savefile.routes.longestRoute, routeLength)
  savefile.routes.totalDistance = savefile.routes.totalDistance + routeLength
  savefile.routes.numberOfRoutes = savefile.routes.numberOfRoutes + 1

  savefile.waypoints.totalWaypoints = savefile.waypoints.totalWaypoints + numWaypoints
  savefile.waypoints.mostNumberOfWaypoints = math.max(savefile.waypoints.mostNumberOfWaypoints, numWaypoints)

  jsonWriteFile("prt/stats.json", savefile, true)
end

--- @return table A list of achievements and their current progress
local function getAchievementStats()
  local data = openSaveFile()

  return {
    singleWps = {
      name = "Single Scenario Waypoints",
      value = data.waypoints.mostNumberOfWaypoints,
      targets = { 10, 20, 80, 160 }
    },
    singleDist = {
      name = "Single Scenario Distance (km)",
      value = math.floor(data.routes.longestRoute * 10) / 10,
      targets = { 2, 4, 16, 32 }
    },
    lifetimeWps = {
      name = "Lifetime Waypoints",
      value = data.waypoints.totalWaypoints,
      targets = { 75, 150, 600, 1200 }
    },
    lifetimeDist = {
      name = "Lifetime Distance (km)",
      value = math.floor(data.routes.totalDistance * 10) / 10,
      targets = { 20, 40, 160, 320 }
    },
    vehiclesUsed = {
      name = "Unique Vehicles Used",
      value = tableComp.lengthOf(data.vehicles.used),
      description = "Configurations of vehicles are allowed - ie. Bolide 320 and Bolide 350 are two different vehicles",
      targets = { 2, 4, 16, 32 }
    }
  }
end

return {
  addScenarioToStats = addScenarioToStats,
  getAchievementStats = getAchievementStats
}