--- Returns the full directory name for the file
--- @param mapName string The name of the map
--- @param filename string The name of the path filename
--- @return string The full directory name for the file
local function directoryFileName(mapName, filename)
  return "prt/"..mapName.."/"..filename
end

--- Returns the list of file names for the given map
--- @param mapName string The name of the map
--- @return table<string, string> The list of file names for that map
local function findExistingFiles(mapName)
  return FS:findFiles("prt/"..mapName.."/", "*", 0, false, false)
end

--- Loads a path from file
--- @param mapName string The name of the map
--- @param filename string The name of the path filename
--- @return table<any, any> - the path (nil if the file could not be opened)
local function load(mapName, filename)
  local directoryName = directoryFileName(mapName, filename)

  local file = readFile(directoryName)
  if not file then return nil end

  return jsonDecode(file)
end

--- Saves a path
--- @param mapName string The name of the map
--- @param filename string The name of the path filename
--- @param routeLength number The length of the route
--- @param routeDriveability number How driveable the route is
--- @param waypoints number The route of waypoints to save (should contain their positions and scale)
--- @param additionalInfo any Any additional info to store in the file
local function save(mapName, filename, routeLength, routeDriveability, waypoints, additionalInfo)
  local data = {
    routeLength = routeLength,
    routeDriveability = routeDriveability,
    waypoints = waypoints
  }

  if additionalInfo then
    data.additionalInfo = additionalInfo
  end

  jsonWriteFile(directoryFileName(mapName, filename), data, true)
end

return {
  findExistingFiles = findExistingFiles,
  load = load,
  save = save
}